@@ -3,12 +3,18 @@ import logging
 
 import voluptuous as vol
 
-from homeassistant.components.geo_location import DOMAIN
+from homeassistant.components.automation import (
+    AutomationActionType,
+    AutomationTriggerInfo,
+)
 from homeassistant.const import CONF_EVENT, CONF_PLATFORM, CONF_SOURCE, CONF_ZONE
-from homeassistant.core import HassJob, callback
+from homeassistant.core import CALLBACK_TYPE, HassJob, HomeAssistant, callback
 from homeassistant.helpers import condition, config_validation as cv
 from homeassistant.helpers.config_validation import entity_domain
 from homeassistant.helpers.event import TrackStates, async_track_state_change_filtered
+from homeassistant.helpers.typing import ConfigType
+
+from . import DOMAIN
 
 # mypy: allow-untyped-defs, no-check-untyped-defs
 
@@ -18,7 +24,7 @@ EVENT_ENTER = "enter"
 EVENT_LEAVE = "leave"
 DEFAULT_EVENT = EVENT_ENTER
 
-TRIGGER_SCHEMA = vol.Schema(
+TRIGGER_SCHEMA = cv.TRIGGER_BASE_SCHEMA.extend(
     {
         vol.Required(CONF_PLATFORM): "geo_location",
         vol.Required(CONF_SOURCE): cv.string,
@@ -35,10 +41,15 @@ def source_match(state, source):
     return state and state.attributes.get("source") == source
 
 
-async def async_attach_trigger(hass, config, action, automation_info):
+async def async_attach_trigger(
+    hass: HomeAssistant,
+    config: ConfigType,
+    action: AutomationActionType,
+    automation_info: AutomationTriggerInfo,
+) -> CALLBACK_TYPE:
     """Listen for state changes based on configuration."""
-    trigger_id = automation_info.get("trigger_id") if automation_info else None
-    source = config.get(CONF_SOURCE).lower()
+    trigger_data = automation_info["trigger_data"]
+    source: str = config[CONF_SOURCE].lower()
     zone_entity_id = config.get(CONF_ZONE)
     trigger_event = config.get(CONF_EVENT)
     job = HassJob(action)
@@ -52,8 +63,7 @@ async def async_attach_trigger(hass, config, action, automation_info):
         if not source_match(from_state, source) and not source_match(to_state, source):
             return
 
-        zone_state = hass.states.get(zone_entity_id)
-        if zone_state is None:
+        if (zone_state := hass.states.get(zone_entity_id)) is None:
             _LOGGER.warning(
                 "Unable to execute automation %s: Zone %s not found",
                 automation_info["name"],
@@ -78,6 +88,7 @@ async def async_attach_trigger(hass, config, action, automation_info):
                 job,
                 {
                     "trigger": {
+                        **trigger_data,
                         "platform": "geo_location",
                         "source": source,
                         "entity_id": event.data.get("entity_id"),
@@ -86,7 +97,6 @@ async def async_attach_trigger(hass, config, action, automation_info):
                         "zone": zone_state,
                         "event": trigger_event,
                         "description": f"geo_location - {source}",
-                        "id": trigger_id,
                     }
                 },
                 event.context,
