@@ -1,7 +1,7 @@
 """Support for Mikrotik routers as device tracker."""
 from __future__ import annotations
 
-import logging
+from typing import Any
 
 from homeassistant.components.device_tracker.config_entry import ScannerEntity
 from homeassistant.components.device_tracker.const import (
@@ -11,17 +11,12 @@ from homeassistant.components.device_tracker.const import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_registry
-from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.update_coordinator import CoordinatorEntity
 import homeassistant.util.dt as dt_util
 
 from .const import DOMAIN
-
-_LOGGER = logging.getLogger(__name__)
-
-# These are normalized to ATTR_IP and ATTR_MAC to conform
-# to device_tracker
-FILTER_ATTRS = ("ip_address", "mac_address")
+from .hub import Device, MikrotikDataUpdateCoordinator
 
 
 async def async_setup_entry(
@@ -30,11 +25,13 @@ async def async_setup_entry(
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up device tracker for Mikrotik component."""
-    hub = hass.data[DOMAIN][config_entry.entry_id]
+    coordinator: MikrotikDataUpdateCoordinator = hass.data[DOMAIN][
+        config_entry.entry_id
+    ]
 
-    tracked: dict[str, MikrotikHubTracker] = {}
+    tracked: dict[str, MikrotikDataUpdateCoordinatorTracker] = {}
 
-    registry = await entity_registry.async_get_registry(hass)
+    registry = entity_registry.async_get(hass)
 
     # Restore clients that is not a part of active clients list.
     for entity in registry.entities.values():
@@ -45,65 +42,69 @@ async def async_setup_entry(
         ):
 
             if (
-                entity.unique_id in hub.api.devices
-                or entity.unique_id not in hub.api.all_devices
+                entity.unique_id in coordinator.api.devices
+                or entity.unique_id not in coordinator.api.all_devices
             ):
                 continue
-            hub.api.restore_device(entity.unique_id)
+            coordinator.api.restore_device(entity.unique_id)
 
     @callback
-    def update_hub():
+    def update_hub() -> None:
         """Update the status of the device."""
-        update_items(hub, async_add_entities, tracked)
+        update_items(coordinator, async_add_entities, tracked)
 
-    async_dispatcher_connect(hass, hub.signal_update, update_hub)
+    config_entry.async_on_unload(coordinator.async_add_listener(update_hub))
 
     update_hub()
 
 
 @callback
-def update_items(hub, async_add_entities, tracked):
+def update_items(
+    coordinator: MikrotikDataUpdateCoordinator,
+    async_add_entities: AddEntitiesCallback,
+    tracked: dict[str, MikrotikDataUpdateCoordinatorTracker],
+):
     """Update tracked device state from the hub."""
-    new_tracked = []
-    for mac, device in hub.api.devices.items():
+    new_tracked: list[MikrotikDataUpdateCoordinatorTracker] = []
+    for mac, device in coordinator.api.devices.items():
         if mac not in tracked:
-            tracked[mac] = MikrotikHubTracker(device, hub)
+            tracked[mac] = MikrotikDataUpdateCoordinatorTracker(device, coordinator)
             new_tracked.append(tracked[mac])
 
     if new_tracked:
         async_add_entities(new_tracked)
 
 
-class MikrotikHubTracker(ScannerEntity):
+class MikrotikDataUpdateCoordinatorTracker(
+    CoordinatorEntity[MikrotikDataUpdateCoordinator], ScannerEntity
+):
     """Representation of network device."""
 
-    def __init__(self, device, hub):
+    def __init__(
+        self, device: Device, coordinator: MikrotikDataUpdateCoordinator
+    ) -> None:
         """Initialize the tracked device."""
+        super().__init__(coordinator)
         self.device = device
-        self.hub = hub
-        self.unsub_dispatcher = None
+        self._attr_name = str(device.name)
+        self._attr_unique_id = device.mac
 
     @property
-    def is_connected(self):
+    def is_connected(self) -> bool:
         """Return true if the client is connected to the network."""
         if (
             self.device.last_seen
             and (dt_util.utcnow() - self.device.last_seen)
-            < self.hub.option_detection_time
+            < self.coordinator.option_detection_time
         ):
             return True
         return False
 
     @property
-    def source_type(self):
+    def source_type(self) -> str:
         """Return the source type of the client."""
         return SOURCE_TYPE_ROUTER
 
-    @property
-    def name(self) -> str:
-        """Return the name of the client."""
-        return self.device.name
-
     @property
     def hostname(self) -> str:
         """Return the hostname of the client."""
@@ -115,42 +116,11 @@ class MikrotikHubTracker(ScannerEntity):
         return self.device.mac
 
     @property
-    def ip_address(self) -> str:
+    def ip_address(self) -> str | None:
         """Return the mac address of the client."""
         return self.device.ip_address
 
     @property
-    def unique_id(self) -> str:
-        """Return a unique identifier for this device."""
-        return self.device.mac
-
-    @property
-    def available(self) -> bool:
-        """Return if controller is available."""
-        return self.hub.available
-
-    @property
-    def extra_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any] | None:
         """Return the device state attributes."""
-        if self.is_connected:
-            return {k: v for k, v in self.device.attrs.items() if k not in FILTER_ATTRS}
-        return None
-
-    async def async_added_to_hass(self):
-        """Client entity created."""
-        _LOGGER.debug("New network device tracker %s (%s)", self.name, self.unique_id)
-        self.unsub_dispatcher = async_dispatcher_connect(
-            self.hass, self.hub.signal_update, self.async_write_ha_state
-        )
-
-    async def async_update(self):
-        """Synchronize state with hub."""
-        _LOGGER.debug(
-            "Updating Mikrotik tracked client %s (%s)", self.entity_id, self.unique_id
-        )
-        await self.hub.request_update()
-
-    async def will_remove_from_hass(self):
-        """Disconnect from dispatcher."""
-        if self.unsub_dispatcher:
-            self.unsub_dispatcher()
+        return self.device.attrs if self.is_connected else None
