@@ -4,8 +4,7 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Callable, Mapping
 from dataclasses import dataclass
-import logging
-from typing import Any, Final, cast
+from typing import Any, cast
 
 from aioshelly.block_device import Block
 import async_timeout
@@ -34,6 +33,7 @@ from .const import (
     BLOCK,
     DATA_CONFIG_ENTRY,
     DOMAIN,
+    LOGGER,
     REST,
     RPC,
     RPC_POLL,
@@ -45,10 +45,9 @@ from .utils import (
     get_rpc_key_instances,
 )
 
-_LOGGER: Final = logging.getLogger(__name__)
 
-
-async def async_setup_entry_attribute_entities(
+@callback
+def async_setup_entry_attribute_entities(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
@@ -64,11 +63,11 @@ async def async_setup_entry_attribute_entities(
     ][BLOCK]
 
     if wrapper.device.initialized:
-        await async_setup_block_attribute_entities(
+        async_setup_block_attribute_entities(
             hass, async_add_entities, wrapper, sensors, sensor_class
         )
     else:
-        await async_restore_block_attribute_entities(
+        async_restore_block_attribute_entities(
             hass,
             config_entry,
             async_add_entities,
@@ -79,7 +78,8 @@ async def async_setup_entry_attribute_entities(
         )
 
 
-async def async_setup_block_attribute_entities(
+@callback
+def async_setup_block_attribute_entities(
     hass: HomeAssistant,
     async_add_entities: AddEntitiesCallback,
     wrapper: BlockDeviceWrapper,
@@ -107,7 +107,7 @@ async def async_setup_block_attribute_entities(
             ):
                 domain = sensor_class.__module__.split(".")[-1]
                 unique_id = f"{wrapper.mac}-{block.description}-{sensor_id}"
-                await async_remove_shelly_entity(hass, domain, unique_id)
+                async_remove_shelly_entity(hass, domain, unique_id)
             else:
                 blocks.append((block, sensor_id, description))
 
@@ -122,7 +122,8 @@ async def async_setup_block_attribute_entities(
     )
 
 
-async def async_restore_block_attribute_entities(
+@callback
+def async_restore_block_attribute_entities(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
@@ -136,7 +137,7 @@ async def async_restore_block_attribute_entities(
     """Restore block attributes entities."""
     entities = []
 
-    ent_reg = await entity_registry.async_get_registry(hass)
+    ent_reg = entity_registry.async_get(hass)
     entries = entity_registry.async_entries_for_config_entry(
         ent_reg, config_entry.entry_id
     )
@@ -160,7 +161,8 @@ async def async_restore_block_attribute_entities(
     async_add_entities(entities)
 
 
-async def async_setup_entry_rpc(
+@callback
+def async_setup_entry_rpc(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
@@ -194,7 +196,7 @@ async def async_setup_entry_rpc(
             ):
                 domain = sensor_class.__module__.split(".")[-1]
                 unique_id = f"{wrapper.mac}-{key}-{sensor_id}"
-                await async_remove_shelly_entity(hass, domain, unique_id)
+                async_remove_shelly_entity(hass, domain, unique_id)
             else:
                 if description.use_polling_wrapper:
                     entities.append(
@@ -209,7 +211,8 @@ async def async_setup_entry_rpc(
     async_add_entities(entities)
 
 
-async def async_setup_entry_rest(
+@callback
+def async_setup_entry_rest(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
@@ -313,12 +316,12 @@ class ShellyBlockEntity(entity.Entity):
 
     async def set_state(self, **kwargs: Any) -> Any:
         """Set block state (HTTP request)."""
-        _LOGGER.debug("Setting state for entity %s, state: %s", self.name, kwargs)
+        LOGGER.debug("Setting state for entity %s, state: %s", self.name, kwargs)
         try:
             async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
                 return await self.block.set_state(**kwargs)
         except (asyncio.TimeoutError, OSError) as err:
-            _LOGGER.error(
+            LOGGER.error(
                 "Setting state for entity %s failed, state: %s, error: %s",
                 self.name,
                 kwargs,
@@ -351,6 +354,11 @@ class ShellyRpcEntity(entity.Entity):
         """Available."""
         return self.wrapper.device.connected
 
+    @property
+    def status(self) -> dict:
+        """Device status by entity key."""
+        return cast(dict, self.wrapper.device.status[self.key])
+
     async def async_added_to_hass(self) -> None:
         """When entity is added to HASS."""
         self.async_on_remove(self.wrapper.async_add_listener(self._update_callback))
@@ -366,7 +374,7 @@ class ShellyRpcEntity(entity.Entity):
 
     async def call_rpc(self, method: str, params: Any) -> Any:
         """Call RPC method."""
-        _LOGGER.debug(
+        LOGGER.debug(
             "Call RPC for entity %s, method: %s, params: %s",
             self.name,
             method,
@@ -376,7 +384,7 @@ class ShellyRpcEntity(entity.Entity):
             async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
                 return await self.wrapper.device.call_rpc(method, params)
         except asyncio.TimeoutError as err:
-            _LOGGER.error(
+            LOGGER.error(
                 "Call RPC for entity %s failed, method: %s, params: %s, error: %s",
                 self.name,
                 method,
@@ -620,6 +628,6 @@ class ShellySleepingBlockAttributeEntity(ShellyBlockAttributeEntity, RestoreEnti
                 self.block = block
                 self.entity_description = description
 
-                _LOGGER.debug("Entity %s attached to block", self.name)
+                LOGGER.debug("Entity %s attached to block", self.name)
                 super()._update_callback()
                 return
