@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-import logging
 
 from zwave_js_server.client import Client as ZwaveClient
 from zwave_js_server.const import CommandClass
@@ -10,36 +9,25 @@ from zwave_js_server.const.command_class.lock import DOOR_STATUS_PROPERTY
 from zwave_js_server.const.command_class.notification import (
     CC_SPECIFIC_NOTIFICATION_TYPE,
 )
+from zwave_js_server.model.driver import Driver
 
 from homeassistant.components.binary_sensor import (
-    DEVICE_CLASS_BATTERY,
-    DEVICE_CLASS_DOOR,
-    DEVICE_CLASS_GAS,
-    DEVICE_CLASS_HEAT,
-    DEVICE_CLASS_LOCK,
-    DEVICE_CLASS_MOISTURE,
-    DEVICE_CLASS_MOTION,
-    DEVICE_CLASS_PLUG,
-    DEVICE_CLASS_PROBLEM,
-    DEVICE_CLASS_SAFETY,
-    DEVICE_CLASS_SMOKE,
-    DEVICE_CLASS_SOUND,
-    DEVICE_CLASS_TAMPER,
     DOMAIN as BINARY_SENSOR_DOMAIN,
+    BinarySensorDeviceClass,
     BinarySensorEntity,
     BinarySensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ENTITY_CATEGORY_DIAGNOSTIC
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import DATA_CLIENT, DOMAIN
 from .discovery import ZwaveDiscoveryInfo
 from .entity import ZWaveBaseEntity
 
-LOGGER = logging.getLogger(__name__)
+PARALLEL_UPDATES = 0
 
 
 NOTIFICATION_SMOKE_ALARM = "1"
@@ -91,101 +79,101 @@ NOTIFICATION_SENSOR_MAPPINGS: tuple[NotificationZWaveJSEntityDescription, ...] =
         # NotificationType 1: Smoke Alarm - State Id's 1 and 2 - Smoke detected
         key=NOTIFICATION_SMOKE_ALARM,
         states=("1", "2"),
-        device_class=DEVICE_CLASS_SMOKE,
+        device_class=BinarySensorDeviceClass.SMOKE,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 1: Smoke Alarm - All other State Id's
         key=NOTIFICATION_SMOKE_ALARM,
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 2: Carbon Monoxide - State Id's 1 and 2
         key=NOTIFICATION_CARBON_MONOOXIDE,
         states=("1", "2"),
-        device_class=DEVICE_CLASS_GAS,
+        device_class=BinarySensorDeviceClass.CO,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 2: Carbon Monoxide - All other State Id's
         key=NOTIFICATION_CARBON_MONOOXIDE,
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 3: Carbon Dioxide - State Id's 1 and 2
         key=NOTIFICATION_CARBON_DIOXIDE,
         states=("1", "2"),
-        device_class=DEVICE_CLASS_GAS,
+        device_class=BinarySensorDeviceClass.GAS,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 3: Carbon Dioxide - All other State Id's
         key=NOTIFICATION_CARBON_DIOXIDE,
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 4: Heat - State Id's 1, 2, 5, 6 (heat/underheat)
         key=NOTIFICATION_HEAT,
         states=("1", "2", "5", "6"),
-        device_class=DEVICE_CLASS_HEAT,
+        device_class=BinarySensorDeviceClass.HEAT,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 4: Heat - All other State Id's
         key=NOTIFICATION_HEAT,
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 5: Water - State Id's 1, 2, 3, 4
         key=NOTIFICATION_WATER,
         states=("1", "2", "3", "4"),
-        device_class=DEVICE_CLASS_MOISTURE,
+        device_class=BinarySensorDeviceClass.MOISTURE,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 5: Water - All other State Id's
         key=NOTIFICATION_WATER,
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 6: Access Control - State Id's 1, 2, 3, 4 (Lock)
         key=NOTIFICATION_ACCESS_CONTROL,
         states=("1", "2", "3", "4"),
-        device_class=DEVICE_CLASS_LOCK,
+        device_class=BinarySensorDeviceClass.LOCK,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 6: Access Control - State Id's 11 (Lock jammed)
         key=NOTIFICATION_ACCESS_CONTROL,
         states=("11",),
-        device_class=DEVICE_CLASS_PROBLEM,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.PROBLEM,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 6: Access Control - State Id 22 (door/window open)
         key=NOTIFICATION_ACCESS_CONTROL,
         off_state="23",
         states=("22", "23"),
-        device_class=DEVICE_CLASS_DOOR,
+        device_class=BinarySensorDeviceClass.DOOR,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 7: Home Security - State Id's 1, 2 (intrusion)
         key=NOTIFICATION_HOME_SECURITY,
         states=("1", "2"),
-        device_class=DEVICE_CLASS_SAFETY,
+        device_class=BinarySensorDeviceClass.SAFETY,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 7: Home Security - State Id's 3, 4, 9 (tampering)
         key=NOTIFICATION_HOME_SECURITY,
         states=("3", "4", "9"),
-        device_class=DEVICE_CLASS_TAMPER,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.TAMPER,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 7: Home Security - State Id's 5, 6 (glass breakage)
         key=NOTIFICATION_HOME_SECURITY,
         states=("5", "6"),
-        device_class=DEVICE_CLASS_SAFETY,
+        device_class=BinarySensorDeviceClass.SAFETY,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 7: Home Security - State Id's 7, 8 (motion)
         key=NOTIFICATION_HOME_SECURITY,
         states=("7", "8"),
-        device_class=DEVICE_CLASS_MOTION,
+        device_class=BinarySensorDeviceClass.MOTION,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 8: Power Management -
@@ -193,55 +181,55 @@ NOTIFICATION_SENSOR_MAPPINGS: tuple[NotificationZWaveJSEntityDescription, ...] =
         key=NOTIFICATION_POWER_MANAGEMENT,
         off_state="2",
         states=("2", "3"),
-        device_class=DEVICE_CLASS_PLUG,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.PLUG,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 8: Power Management -
         # State Id's 6, 7, 8, 9 (power status)
         key=NOTIFICATION_POWER_MANAGEMENT,
         states=("6", "7", "8", "9"),
-        device_class=DEVICE_CLASS_SAFETY,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.SAFETY,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 8: Power Management -
         # State Id's 10, 11, 17 (Battery maintenance status)
         key=NOTIFICATION_POWER_MANAGEMENT,
         states=("10", "11", "17"),
-        device_class=DEVICE_CLASS_BATTERY,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.BATTERY,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 9: System - State Id's 1, 2, 3, 4, 6, 7
         key=NOTIFICATION_SYSTEM,
         states=("1", "2", "3", "4", "6", "7"),
-        device_class=DEVICE_CLASS_PROBLEM,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.PROBLEM,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 10: Emergency - State Id's 1, 2, 3
         key=NOTIFICATION_EMERGENCY,
         states=("1", "2", "3"),
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 14: Siren
         key=NOTIFICATION_SIREN,
         states=("1",),
-        device_class=DEVICE_CLASS_SOUND,
+        device_class=BinarySensorDeviceClass.SOUND,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 18: Gas
         key=NOTIFICATION_GAS,
         states=("1", "2", "3", "4"),
-        device_class=DEVICE_CLASS_GAS,
+        device_class=BinarySensorDeviceClass.GAS,
     ),
     NotificationZWaveJSEntityDescription(
         # NotificationType 18: Gas
         key=NOTIFICATION_GAS,
         states=("6",),
-        device_class=DEVICE_CLASS_PROBLEM,
+        device_class=BinarySensorDeviceClass.PROBLEM,
     ),
 )
 
@@ -251,17 +239,17 @@ PROPERTY_SENSOR_MAPPINGS: dict[str, PropertyZWaveJSEntityDescription] = {
     DOOR_STATUS_PROPERTY: PropertyZWaveJSEntityDescription(
         key=DOOR_STATUS_PROPERTY,
         on_states=("open",),
-        device_class=DEVICE_CLASS_DOOR,
+        device_class=BinarySensorDeviceClass.DOOR,
     ),
 }
 
 
 # Mappings for boolean sensors
-BOOLEAN_SENSOR_MAPPINGS: dict[str, BinarySensorEntityDescription] = {
+BOOLEAN_SENSOR_MAPPINGS: dict[int, BinarySensorEntityDescription] = {
     CommandClass.BATTERY: BinarySensorEntityDescription(
         key=str(CommandClass.BATTERY),
-        device_class=DEVICE_CLASS_BATTERY,
-        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+        device_class=BinarySensorDeviceClass.BATTERY,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
 }
 
@@ -277,6 +265,8 @@ async def async_setup_entry(
     @callback
     def async_add_binary_sensor(info: ZwaveDiscoveryInfo) -> None:
         """Add Z-Wave Binary Sensor."""
+        driver = client.driver
+        assert driver is not None  # Driver is ready before platforms are loaded.
         entities: list[BinarySensorEntity] = []
 
         if info.platform_hint == "notification":
@@ -308,22 +298,26 @@ async def async_setup_entry(
 
                 entities.append(
                     ZWaveNotificationBinarySensor(
-                        config_entry, client, info, state_key, notification_description
+                        config_entry, driver, info, state_key, notification_description
                     )
                 )
-        elif info.platform_hint == "property" and (
-            property_description := PROPERTY_SENSOR_MAPPINGS.get(
-                info.primary_value.property_name
+        elif (
+            info.platform_hint == "property"
+            and info.primary_value.property_name
+            and (
+                property_description := PROPERTY_SENSOR_MAPPINGS.get(
+                    info.primary_value.property_name
+                )
             )
         ):
             entities.append(
                 ZWavePropertyBinarySensor(
-                    config_entry, client, info, property_description
+                    config_entry, driver, info, property_description
                 )
             )
         else:
             # boolean sensor
-            entities.append(ZWaveBooleanBinarySensor(config_entry, client, info))
+            entities.append(ZWaveBooleanBinarySensor(config_entry, driver, info))
 
         async_add_entities(entities)
 
@@ -342,11 +336,11 @@ class ZWaveBooleanBinarySensor(ZWaveBaseEntity, BinarySensorEntity):
     def __init__(
         self,
         config_entry: ConfigEntry,
-        client: ZwaveClient,
+        driver: Driver,
         info: ZwaveDiscoveryInfo,
     ) -> None:
         """Initialize a ZWaveBooleanBinarySensor entity."""
-        super().__init__(config_entry, client, info)
+        super().__init__(config_entry, driver, info)
 
         # Entity class attributes
         self._attr_name = self.generate_name(include_value_name=True)
@@ -369,13 +363,13 @@ class ZWaveNotificationBinarySensor(ZWaveBaseEntity, BinarySensorEntity):
     def __init__(
         self,
         config_entry: ConfigEntry,
-        client: ZwaveClient,
+        driver: Driver,
         info: ZwaveDiscoveryInfo,
         state_key: str,
         description: NotificationZWaveJSEntityDescription | None = None,
     ) -> None:
         """Initialize a ZWaveNotificationBinarySensor entity."""
-        super().__init__(config_entry, client, info)
+        super().__init__(config_entry, driver, info)
         self.state_key = state_key
         if description:
             self.entity_description = description
@@ -404,12 +398,12 @@ class ZWavePropertyBinarySensor(ZWaveBaseEntity, BinarySensorEntity):
     def __init__(
         self,
         config_entry: ConfigEntry,
-        client: ZwaveClient,
+        driver: Driver,
         info: ZwaveDiscoveryInfo,
         description: PropertyZWaveJSEntityDescription,
     ) -> None:
         """Initialize a ZWavePropertyBinarySensor entity."""
-        super().__init__(config_entry, client, info)
+        super().__init__(config_entry, driver, info)
         self.entity_description = description
         self._attr_name = self.generate_name(include_value_name=True)
 
